unit jclWindows;
{$I jcl.inc}

interface
uses windows;

type
  PCancelWaitableTimer = function (hHandle: Cardinal) : LongBool of object; stdcall;

  PCreateWaitableTimer = function (lpTimerAttributes: PSecurityAttributes; bManualReset: BOOL; lpTimerName: PAnsiChar): THandle of object; stdcall;

  PInitializeCriticalSectionAndSpinCount = function (var lpCriticalSection: TRTLCriticalSection; dwSpinCount: DWORD): BOOL of Object; stdcall;

  PGetFileAttributesEx = function (lpFileName: PAnsiChar; fInfoLevelId: TGetFileExInfoLevels; lpFileInformation: Pointer): BOOL of object; stdcall;

  PSetCriticalSectionSpinCount = function (var lpCriticalSection: TRTLCriticalSection; dwSpinCount: DWORD): DWORD of object; stdcall;

  PSetWaitableTimer = function (hTimer: THandle; var lpDueTime: TLargeInteger;
    lPeriod: Longint; pfnCompletionRoutine: TFNTimerAPCRoutine;
    lpArgToCompletionRoutine: Pointer; fResume: BOOL): BOOL of object; stdcall;

  PSignalObjectAndWait = function (hObjectToSignal: THandle; hObjectToWaitOn: THandle;
  dwMilliseconds: DWORD; bAlertable: BOOL): DWORD of object; stdcall;

  PTryEnterCriticalSection = function (var lpCriticalSection: TRTLCriticalSection): BOOL of object; stdcall;

  POpenWaitableTimer = function  (dwDesiredAccess: DWORD; bInheritHandle: BOOL;
    lpTimerName: PAnsiChar): THandle of object; stdcall;
{
  PPSID = ^PSID;
  SE_OBJECT_TYPE = (
    SE_UNKNOWN_OBJECT_TYPE,
    SE_FILE_OBJECT,
    SE_SERVICE,
    SE_PRINTER,
    SE_REGISTRY_KEY,
    SE_LMSHARE,
    SE_KERNEL_OBJECT,
    SE_WINDOW_OBJECT,
    SE_DS_OBJECT,
    SE_DS_OBJECT_ALL,
    SE_PROVIDER_DEFINED_OBJECT,
    SE_WMIGUID_OBJECT
  );

  PSetNamedSecurityInfoW = function (pObjectName: PWideChar; ObjectType: SE_OBJECT_TYPE;
    SecurityInfo: SECURITY_INFORMATION; ppsidOwner, ppsidGroup: PPSID; ppDacl,
    ppSacl: PACL): DWORD of object; stdcall;

  PAdjustTokenPrivileges = function (TokenHandle: THandle; DisableAllPrivileges: BOOL;
    const NewState: TTokenPrivileges; BufferLength: DWORD;
    PreviousState: PTokenPrivileges; ReturnLength: PDWORD): BOOL of object; stdcall;

  HINTERNET = Pointer; 
  PFtpGetFileSize = function (hFile: HINTERNET; lpdwFileSizeHigh: LPDWORD): DWORD of object; stdcall;
}
var
  CancelWaitableTimer: PCancelWaitableTimer;
  CreateWaitableTimer: PCreateWaitableTimer;
  InitializeCriticalSectionAndSpinCount: PInitializeCriticalSectionAndSpinCount;
  GetFileAttributesEx: PGetFileAttributesEx;
  OpenWaitableTimer : POpenWaitableTimer;
  SetCriticalSectionSpinCount: PSetCriticalSectionSpinCount;
  SetWaitableTimer: PSetWaitableTimer;
  SignalObjectAndWait: PSignalObjectAndWait;
  TryEnterCriticalSection: PTryEnterCriticalSection;
{
  SetNamedSecurityInfo: PSetNamedSecurityInfoW;
  AdjustTokenPrivileges: PAdjustTokenPrivileges;
  FtpGetFileSize: PFtpGetFileSize;
}  
implementation
uses JclDynamicLoad;

const
  Kernel32Dll = 'kernel32.dll';
{
  AdvAPI32Dll = 'advapi32.dll';
  WinetDll = 'wininet.dll';
}
var
   kernel32{, advapi32, wininet}: HINST;
   KeepDLLInMemory: boolean;

initialization

  kernel32 := LoadLibrary (Kernel32Dll);
  if (kernel32 <> 0) then begin
    KeepDLLInMemory := LoadFunction (kernel32, 'KERNEL32.CancelWaitableTimer', @@CancelWaitableTimer);
    KeepDLLInMemory := LoadFunction (kernel32, 'KERNEL32.CreateWaitableTimerA', @@CreateWaitableTimer) or KeepDLLInMemory;
    KeepDLLInMemory := LoadFunction (kernel32, 'KERNEL32.InitializeCriticalSectionAndSpinCount', @@InitializeCriticalSectionAndSpinCount) or KeepDLLInMemory;
    KeepDLLInMemory := LoadFunction (kernel32, 'KERNEL32.GetFileAttributesExA', @@GetFileAttributesEx) or KeepDLLInMemory;
    KeepDLLInMemory := LoadFunction (kernel32, 'KERNEL32.OpenWaitableTimerA', @@OpenWaitableTimer) or KeepDLLInMemory;
    KeepDLLInMemory := LoadFunction (kernel32, 'KERNEL32.SetCriticalSectionSpinCount', @@SetCriticalSectionSpinCount) or KeepDLLInMemory;
    KeepDLLInMemory := LoadFunction (kernel32, 'KERNEL32.SetWaitableTimer', @@SetWaitableTimer) or KeepDLLInMemory;
    KeepDLLInMemory := LoadFunction (kernel32, 'KERNEL32.SignalObjectAndWait', @@SignalObjectAndWait) or KeepDLLInMemory;
    KeepDLLInMemory := LoadFunction (kernel32, 'KERNEL32.TryEnterCriticalSection', @@TryEnterCriticalSection) or KeepDLLInMemory;

    if not KeepDLLInMemory then begin
      FreeLibrary (kernel32);
      kernel32 := 0;
    end;
  end;
{
  advapi32 := LoadLibrary (AdvAPI32Dll);
  if (advapi32 <> 0) then begin
    KeepDLLInMemory := LoadFunction (advapi32, 'ADVAPI32.SetNamedSecurityInfoW', @@SetNamedSecurityInfo);
    KeepDLLInMemory := LoadFunction (advapi32, 'ADVAPI32.AdjustTokenPrivileges', @@AdjustTokenPrivileges) or KeepDLLInMemory;

    if not KeepDLLInMemory then begin
      FreeLibrary (advapi32);
      advapi32 := 0;
    end;
  end;

  wininet := LoadLibrary (WinetDll);
  if (wininet <> 0) then begin
    KeepDLLInMemory := LoadFunction (wininet, 'WININET.FtpGetFileSize', @@FtpGetFileSize);

    if not KeepDLLInMemory then begin
      FreeLibrary (wininet);
      wininet := 0;
    end;
  end;
}
finalization
  if (kernel32 <> 0) then FreeLibrary (kernel32);
//  if (advapi32 <> 0) then FreeLibrary (advapi32);
//  if (wininet <> 0) then FreeLibrary (wininet);
end.
